;************************************************************************************************
;*                                  Autoom.ASM                                                 *
;*   :                                                       *
;*              : Copyright (c) 2013 www.v-50m.narod.ru                                         *
;*              : All Rights Reserved                                                           *
;*          : 26.08.2013                                                                    *
;*        : 1.0                                                                           *
;*      :                                            *
;*                                                                 *
;*                       PIC16F628 			*
;*		    ()							                *                                        
;************************************************************************************************

#DEFINE PIC16F628

#INCLUDE <p16f628a.inc>


;  ----------------------------------------------------------------------------------

    __CONFIG    _CP_OFF & _WDT_ON & _PWRTE_ON & _XT_OSC & _LVP_OFF & _MCLRE_ON & _BODEN_OFF & _DATA_CP_OFF
    __IDLOCS    0xAC10

;---------------------------   -------------------------
#DEFINE		P_SIGNAL	PORTB , 00	;     
#DEFINE		OUT_SIGNAL	PORTB , 01	;   

#DEFINE		N_SIGNAL	PORTB , 03	;     
#DEFINE		MAP		PORTB , 04	;  
#DEFINE		LOCK_ON		PORTB , 05	;  - ""

#DEFINE		IN_LED		PORTA , 02	;  ""
#DEFINE		LOCK_LED	PORTA , 03	;  ""

;---------------------------  ----------------------------------------------------------
CBLOCK	20h
W_temp
Status_temp
Ignition
DelayCounter : 3
ENDC

CBLOCK 70h
X				
Y
T : 2			
Temp : 4
Tmin : 2
Tmax : 2
Tnp : 2			
ENDC
	
#DEFINE		LowRPM		Ignition , 00
#DEFINE		Spark		Ignition , 01
#DEFINE		Break		Ignition , 02
#DEFINE		SkipPulse	Ignition , 03
#DEFINE		CalibrationEnable		Ignition , 04

;---------------------------  ----------------------------------------------------------
PORTA_RESET		EQU	B'00000'
PORTB_RESET		EQU	B'00000000'
TRISA_RESET		EQU	B'00000'
TRISB_RESET		EQU	B'00111001'

INTCON_RESET		EQU	B'11000000'
T1CON_RESET		EQU	B'00110000'
CCP1CON_RESET		EQU	B'00000000'
 ;CCP1CON_RESET		EQU	B'00001010'
T2CON_RESET		EQU	B'00000000'
PR2_RESET		EQU	.100
PIE1_RESET		EQU	B'00000010'
 ;PIE1_RESET		EQU	B'00000110'
OPTION_RESET		EQU	B'10000000'

IGNITION_RESET		EQU	B'00011001'
INPUT_DELAY		EQU	.50	;7+3x




;================================================================================================
; ---------------------------------------   -----------------------------------------
;================================================================================================   
     ORG 0000h
	goto Start

;================================================================================================
; ---------------------------------------   -------------------------------------
;================================================================================================
    ORG	0004h
	btfsc	PIR1, TMR2IF
	goto	TMR2_interrupt

	goto $		;     ,    
			; .    WDT
;================================================================================================
; ---------------------------------------   TMR2 ------------------------------------
;================================================================================================
TMR2_interrupt
	bcf	OUT_SIGNAL
	movwf	W_temp
	swapf	STATUS, w
	movwf	Status_temp
	bcf	T2CON , TMR2ON
	clrf	TMR2
	bcf	PIR1 , TMR2IF
	swapf	Status_temp, w
	movwf	STATUS
	swapf	W_temp, f
	swapf	W_temp, w
	retfie

;================================================================================================
; -----------------        ----------------------------
;================================================================================================
Start	
	clrwdt
	movlw	PORTA_RESET	;     
	movwf	PORTA		;
	movlw	PORTB_RESET	;
	movwf	PORTB		;

	movlw	0x07		;   
	movwf	CMCON		;

	bsf	STATUS , RP0		
	movlw	TRISA_RESET	;   
	movwf	TRISA		;
	movlw	TRISB_RESET	;
	movwf	TRISB		;
	movlw	PIE1_RESET	;
	movwf	PIE1		;
	movlw	OPTION_RESET	;
	movwf	OPTION_REG	;
	movlw	PR2_RESET	;
	movwf	PR2		;

	movlw	0x78		;    EEPROM
	movwf	EEADR		;
	bsf	EECON1, RD	;
	movf	EEDATA, w	;
	movwf	Tmax+1		;
	incf	EEADR, f	;
	bsf	EECON1, RD	;
	movf	EEDATA, w	;
	movwf	Tmax+0		;
	incf	EEADR, f	;
	bsf	EECON1, RD	;
	movf	EEDATA, w	;
	movwf	Tmin+1		;
	incf	EEADR, f	;
	bsf	EECON1, RD	;
	movf	EEDATA, w	;
	movwf	Tmin+0		;
	bcf	STATUS, RP0	

	clrf	TMR1L
	clrf	TMR1H
	clrf	CCPR1L
	clrf	CCPR1H
	movlw	CCP1CON_RESET
	movwf	CCP1CON
	movlw	T1CON_RESET	;  
	movwf	T1CON		;
	movlw	T2CON_RESET	; 
	movwf	T2CON		;

	movlw	IGNITION_RESET
	movwf	Ignition

	movlw	INTCON_RESET	;  
	movwf	INTCON		;

	btfss	LOCK_ON
	bsf	LOCK_LED

;================================================================================================
; ------------------------------------   -----------------------------------
;================================================================================================
Main
	clrwdt
	btfsc	LowRPM
	call	Invert_Led
	
	btfss	INTCON, INTF
	goto	Main

	btfsc	N_SIGNAL
	goto	PP_interrupt
	goto	NP_interrupt

;================================================================================================
; ---------------------------------------   PP --------------------------------------
;================================================================================================
PP_interrupt
	call	Delay
	bcf	INTCON, INTF
	btfsc	P_SIGNAL
	goto	Main

	bcf	INTCON, GIE
	bcf	T1CON, TMR1ON
	movf	TMR1H, w	
	movwf	Tnp+1		;     Tnp 
	movf	TMR1L, w
	movwf	Tnp+0
	bsf	T1CON, TMR1ON
	bsf	INTCON, GIE

	bsf	OUT_SIGNAL
	bsf	T2CON, TMR2ON
	bsf	Spark
	bsf	SkipPulse

	goto	Main

;================================================================================================
; ---------------------------------------   NP --------------------------------------
;================================================================================================
NP_interrupt
	call	Delay
	bcf	INTCON, INTF
	btfsc	N_SIGNAL
	goto	Main	
	
	btfss	SkipPulse
	goto	Change_const
	bcf	SkipPulse
	goto	Main

Change_const
	bcf	INTCON, GIE
	bcf	T1CON, TMR1ON
	movf	TMR1H, w	
	movwf	T+1		;     T ...
	movf	TMR1L, w
	movwf	T+0
	clrf	TMR1L		; ...   
	bsf	T1CON, TMR1ON
	clrf	TMR1H		;
	bsf	INTCON, GIE
	
	bcf	Break
	bcf	LowRPM
	bcf	Spark

	btfsc	PIR1, TMR1IF
	bsf	LowRPM
	bcf	PIR1, TMR1IF

CompareTmax
	movf	Tmax+0, w 	;  T  Tmax
	subwf	T+0, w		;
	movf	Tmax+1, w	;
	btfss	STATUS, C	;
	incfsz	Tmax+1, w 	;
	subwf	T+1, w		;
	btfss	STATUS, C	
	goto	$+2
	bsf	LowRPM		;  ,  T > Tmax (  )

	btfss	LowRPM
	bsf	IN_LED
	goto	Main	

;================================================================================================
; --------------------------------  -------------------------------------
;================================================================================================
Invert_Led
	decfsz	DelayCounter+0, f
	return
	decfsz	DelayCounter+1, f
	return
	clrw
	btfss   IN_LED
	addlw	0x01
        btfsc   STATUS    , Z
	bcf	IN_LED
        btfss   STATUS    , Z
	bsf	IN_LED
	return
;================================================================================================
; --------------------------------  -------------------------------------
;================================================================================================
Delay
	movlw	INPUT_DELAY
	movwf	DelayCounter+2
DelayLoop
	decfsz	DelayCounter+2, f
	goto	DelayLoop
	return

	ORG 2178h
     DATA     0xB           ; 78     Tmaxh
     DATA     0xB8           ; 79     Tmaxl
     DATA     0x9           ; 7A     Tminh
     DATA     0xC4           ; 7B     Tminl

	 END
	
